VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCards"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'Copyright (C) 2004 Kristian. S.Stangeland

'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; either version 2
'of the License, or (at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Private Type Addresses
    AddressName As String
    AddressDescription As String
End Type

Private Type Card
    Name As String
    AddressCount As Long
    Addresses() As Addresses
End Type

Private Type FileHeader
    AdminPass As String
    CardCount As Long
    Cards() As Card
End Type

' Public events
Event CardAdded()
Event CardRemoved()
Event AddressAdded(ByVal Card As Long)

' Private variables
Private FileData As FileHeader

' Public properties
Public FilePath As String

Public Function AddCard(sName As String) As Long

    ' Increse the array
    FileData.CardCount = FileData.CardCount + 1
    
    ' Reallocate the array
    ReDim Preserve FileData.Cards(FileData.CardCount)
    
    FileData.Cards(FileData.CardCount).Name = sName
    FileData.Cards(FileData.CardCount).AddressCount = -1

    ' A card is added, raise the event
    RaiseEvent CardAdded

    ' Return the index of the created object
    AddCard = FileData.CardCount

End Function

Public Sub DeleteCard(ByVal Index As Long)

    Dim Tell As Long
    
    For Tell = Index To FileData.CardCount - 1
        LSet FileData.Cards(Tell) = FileData.Cards(Tell + 1)
    Next
    
    If FileData.CardCount <= 0 Then
        FileData.CardCount = -1
        Erase FileData.Cards
    Else
        FileData.CardCount = FileData.CardCount - 1
        ReDim Preserve FileData.Cards(FileData.CardCount)
    End If

    ' A card is removed, raise the event
    RaiseEvent CardRemoved

End Sub

Public Function AddAddress(ByVal Card As Long, Optional sAddress As String, Optional sDescription As String) As Long

    If IsCard(Card) Then
    
        ' Increse the size
        FileData.Cards(Card).AddressCount = FileData.Cards(Card).AddressCount + 1
        
        ' Reallocate the array
        ReDim Preserve FileData.Cards(Card).Addresses(FileData.Cards(Card).AddressCount)
        
        FileData.Cards(Card).Addresses(FileData.Cards(Card).AddressCount).AddressDescription = sDescription
        FileData.Cards(Card).Addresses(FileData.Cards(Card).AddressCount).AddressName = sAddress
        
        ' Raise the event
        RaiseEvent AddressAdded(Card)
        
        ' Return the index
        AddAddress = FileData.Cards(Card).AddressCount
    
    End If

End Function

Public Function DeleteAddress(ByVal Card As Long, ByVal Index As Long) As Long

    Dim Tell As Long

    If IsCard(Card) Then
    
        For Tell = Index To FileData.Cards(Card).AddressCount - 1
            LSet FileData.Cards(Card).Addresses(Index) = FileData.Cards(Card).Addresses(Index)
        Next
        
        If FileData.Cards(Card).AddressCount <= 0 Then
            FileData.Cards(Card).AddressCount = -1
            Erase FileData.Cards(Card).Addresses
        Else
            FileData.Cards(Card).AddressCount = FileData.Cards(Card).AddressCount - 1
            ReDim Preserve FileData.Cards(Card).Addresses(FileData.Cards(Card).AddressCount)
        End If
        
    End If

End Function

Public Sub SaveCards(sFile As String)

    Dim Free As Long
    
    Free = FreeFile
    
    If Dir(sFile) <> "" Then
        Kill sFile
    End If
    
    Open sFile For Binary As #Free
        Put #Free, , FileData
    Close #Free

End Sub

Public Sub LoadCards(sFile As String)

    Dim Free As Long
    
    Free = FreeFile

    If Dir(sFile) <> "" Then
        Open sFile For Binary As #Free
            Get #Free, , FileData
        Close #Free
        
        ' Set this as the current file
        FilePath = sFile
    Else
        ' Remove the current file
        FilePath = ""
    
        MsgBox "Kunne ikke pne filen '" & sFile & "'", vbCritical, "Error"
    End If
    
    ' Update the cards
    RaiseEvent CardAdded
    
End Sub

Public Function IsAddress(ByVal Card As Long, ByVal Index As Long) As Boolean

    On Error Resume Next
    If IsCard(Card) Then
        IsAddress = CBool(Index >= 0 And Index <= FileData.Cards(Card).AddressCount)
    End If
    
End Function

Public Function IsCard(ByVal Index As Long) As Boolean

    IsCard = CBool(Index >= 0 And Index <= FileData.CardCount)
    
End Function

Public Property Get CardName(ByVal Card As Long) As String
    
    On Error Resume Next
    If IsCard(Card) Then
        CardName = FileData.Cards(Card).Name
    End If

End Property

Public Property Let CardName(ByVal Card As Long, ByVal vNewValue As String)

    On Error Resume Next
    If IsCard(Card) Then
        FileData.Cards(Card).Name = vNewValue
    End If

End Property

Public Property Get AddressName(ByVal Card As Long, ByVal Index As Long) As String

    On Error Resume Next
    If IsAddress(Card, Index) Then
        AddressName = FileData.Cards(Card).Addresses(Index).AddressName
    End If

End Property

Public Property Let AddressName(ByVal Card As Long, ByVal Index As Long, ByVal vNewValue As String)

    On Error Resume Next
    If IsAddress(Card, Index) Then
        FileData.Cards(Card).Addresses(Index).AddressName = vNewValue
    End If

End Property

Public Property Get AddressDescription(ByVal Card As Long, ByVal Index As Long) As String

    On Error Resume Next
    If IsAddress(Card, Index) Then
        AddressDescription = FileData.Cards(Card).Addresses(Index).AddressDescription
    End If

End Property

Public Property Get AddressCount(ByVal Card As Long) As Long

    On Error Resume Next
    If IsCard(Card) Then
        AddressCount = FileData.Cards(Card).AddressCount
    End If

End Property

Public Property Let AddressDescription(ByVal Card As Long, ByVal Index As Long, ByVal vNewValue As String)

    On Error Resume Next
    If IsAddress(Card, Index) Then
        FileData.Cards(Card).Addresses(Index).AddressDescription = vNewValue
    End If

End Property

Private Sub Class_Initialize()

    ' This is always -1 in the beginning
    FileData.CardCount = -1

End Sub

Public Property Get AdminPass() As String

    AdminPass = FileData.AdminPass

End Property

Public Property Let AdminPass(ByVal sNewValue As String)

    FileData.AdminPass = sNewValue

End Property

Public Property Get CardCount() As Long

    CardCount = FileData.CardCount

End Property
